---------------------------------------------------------------------------
--
-- CreateMetricsElements
--
-- Create elements according to #MetricsData temporary table.
-- Elements are created row-by-row. After each row, this script
-- is called recursively using the --#Run command.
--
-- Input: 	#_Arg1 			Contains [CurrentDataRow] column that maintains
--							the number of rows processed so far.
--			#SessionData 	MEA web service session identifier.
-- 			#ModelId		Id of the target metrics model (SC.12345)
--			#SubScriptId	Id of this script (to allow recursive calls)
--			#NumberOfFailures Contains the number of failures contained so far.
--							If this temporary table doesn't exists, it is created
--							at the beginning. The value is increased when encountering
--							an error.
--			#MetricsData	A temporary table containing the data to be loaded.
--							The table must contain the following columns (with types)
--				* Id (Int, Not null, Identity(1,1)): contains a unique identity for the data row.
--				* Element (VARCHAR(80)): The name of the element to be created.
--				* Typename (VARCHAR(30)): Type name of the element, e.g. "Scorecard" or "Measure".
--				* Parent (VARCHAR(80)): Name of the parent element; must exists in the #MetricsData table!
--
--				#MetricsData may optionally contain any number of other columns, such
--				as 'Symbol' or 'Description' used for setting Metrics identifier and element description,
--				correspondingly. An error will be thrown, if column doesn't correspond to an attribute
--				that can be set for the element.
--
--				Example SQL create declaration for #MetricsData (with two optional columns 'Symbol' and 'Description':
--				CREATE TABLE #ELEMENTS
--				(
--					Id Int NOT NULL IDENTITY(1,1),
--					Element VARCHAR(80),
--					Typename VARCHAR(30),
--					Parent VARCHAR(80),
--					Description VARCHAR(8000),
--					Symbol VARCHAR(200)
--				);
--
---------------------------------------------------------------------------

-- This script can be only called from another script.
(SELECT 'MessageText', 'Temporary table #ModelId must be set before calling CreateMetricsElements.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#ModelId') IS NOT NULL,'0','1'))
--#Exit 

-- SessionData must exists.
(SELECT 'MessageText', 'Can not find session data (#SessionData temporary table not set)!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#SessionData') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', '#SubScriptId temporary table not set)!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#SubScriptId') IS NOT NULL,'0','1'))
--#Exit 

-- Create a temporary table holding number of failures, if it doesn't exist yet.
IF OBJECT_ID('tempdb..#NumberOfFailures') IS NULL
	SELECT 0 AS VAL INTO #NumberOfFailures;

---------------------------------------------------------------------------
---- Create SC element
---------------------------------------------------------------------------

DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @ModelId AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);

SELECT @SessionString = [SES_ID] FROM #SessionData;
SELECT @ModelId = [VAL] FROM #ModelId;

-- print 'CreateMetricsElements: SessionString: [' + @SessionString + ']'

-- Create #MetricsRowCreate temporary table conforming with #MetricsData structure
IF OBJECT_ID('tempdb..#MetricsRowCreate') IS NULL
BEGIN
	SELECT TOP 1 *
	INTO #MetricsRowCreate
	FROM #MetricsData m
	ORDER BY Id
END

-- Clear #MetricsRowCreate
DELETE FROM #MetricsRowCreate

-- Read one row of data from #MetricsData to #MetricsRowCreate
INSERT INTO #MetricsRowCreate
SELECT TOP 1 * FROM #MetricsData ORDER BY Id

-- Remove the row read from #MetricsData
DELETE FROM #MetricsData WHERE Id=(SELECT Id FROM #MetricsRowCreate)

---------------------------------------------------------
---------------------------------------------------------
-- Insert default parent id here, if required!
-- Default parent identifier is model specific and is in
-- form 'SC.12345.56789'.
---------------------------------------------------------
---------------------------------------------------------
DECLARE @DefaultParentId AS NVARCHAR(MAX)
SET @DefaultParentId = '' 

IF OBJECT_ID('tempdb..#ElementIdMap') IS NULL
	CREATE TABLE #ElementIdMap
	(
		ElementName VARCHAR(80),
		ElementId VARCHAR(80)
	);

-- Get column names for #MetricsRowCreate table
DECLARE @Attributes AS NVARCHAR(MAX);
SET @Attributes = '[';

IF OBJECT_ID('tempdb..#ColNames') IS NULL
	CREATE TABLE #ColNames
	(
		Name VARCHAR(MAX)
	);
INSERT INTO #ColNames
 SELECT name FROM tempdb.sys.columns 
	WHERE [object_id] = OBJECT_ID('tempdb..#MetricsRowCreate')
	AND (name <> 'Id') AND (name <> 'Element') AND (name <> 'Typename') AND (name <> 'Parent');

DECLARE @rowsdone AS INT;
SET @rowsdone = 0;
DECLARE @colname AS NVARCHAR(MAX);
SELECT TOP 1 @colname = Name FROM #ColNames
WHILE (@@rowcount > 0)
BEGIN
	DECLARE @colvalue AS NVARCHAR(MAX);
	DECLARE @sql AS NVARCHAR(MAX);
	SET @sql = N'SELECT @colvalue = CONVERT(NVARCHAR(MAX),[' + @colname + ']) FROM #MetricsRowCreate'
--	print 'SQL: ' + @sql
	EXEC sp_executesql @sql, N'@colvalue VARCHAR(MAX) out', @colvalue out
	IF @colvalue IS NOT NULL
		BEGIN
			SET @Attributes = @Attributes
				+ IIF(@rowsdone > 0, ',','')
				+ '["' 
				+ IIF(@colname IS NOT NULL, @colname, '""') + '","' 
				+ IIF(@colvalue IS NOT NULL, @colvalue, '""') + '"]';
			SET @rowsdone = @rowsdone + 1
		END
	DELETE FROM #ColNames WHERE Name = @colname
	SELECT TOP 1 @colname = Name FROM #ColNames
END

SET @Attributes = @Attributes + ']';

-- print 'Attributes: ' + @Attributes

SELECT @Body = '{"sessionId":"' + @SessionString + '",' +
	'"namespaceId":"' + @ModelId + '",' +
	'"objectTypeName":"' + CONVERT(NVARCHAR(30),M.Typename) + '",' +
	'"name":"' + CONVERT(NVARCHAR(80),M.Element) + '", ' +
 	'"parentIds":"' + IIF(E.ElementId IS NOT NULL, CONVERT(NVARCHAR(80),E.ElementId), CONVERT(NVARCHAR(80), @DefaultParentId)) + '",' +
	'"attributes":' + @Attributes + 
	',"options":""}'
FROM #MetricsRowCreate M LEFT OUTER JOIN #ElementIdMap E
ON M.[Parent] = E.[ElementName]

print 'CreateMetricsElements: Body: [' + @Body + ']'

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/CreateObject') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*')
--#CallWebService

print @_ResponseText

---------------------------------------------------------------------------
---- STORE ID of the created element into #ElementId
---------------------------------------------------------------------------
DECLARE @elementId NVARCHAR(MAX);
DECLARE @pos INT;
DECLARE @ResponseText NVARCHAR(MAX) = @_ResponseText

-- Extract session id
SET @pos = PATINDEX('%"d":"%',@ResponseText);
IF @pos > 0
BEGIN
  SET @elementId = SUBSTRING(@ResponseText, @pos + 5, LEN(@ResponseText))
  SET @pos = PATINDEX('%"%',@elementId);
  IF @pos > 0
  BEGIN
    SET @elementId = SUBSTRING(@elementId, 0, @pos)
--    PRINT 'Created element ID: ' + @elementId;
  END;
END;

-- Store the element id to the #ElementIdMap table
INSERT INTO #ElementIdMap(ElementName, ElementId)
SELECT CONVERT(NVARCHAR(80), [Element]), CONVERT(NVARCHAR(80),@elementId)
FROM #MetricsRowCreate;

-- Get current data row
DECLARE @CurrentDataRow AS INT;
IF OBJECT_ID('tempdb..#_Arg1') IS NOT NULL
 SELECT @CurrentDataRow = CONVERT(INT, [CurrentDataRow]) FROM #_Arg1;
ELSE
 SET @CurrentDataRow = 1;

-- If CallWebService was successful continue, else exit
IF CONVERT(INT, @_ResponseSuccess) = 1
 GOTO CreateObjectSuccess
ELSE
 BEGIN
  print 'CreateObject failed at input row ' + CONVERT(VARCHAR(10), @CurrentDataRow) + ' : ' + @_ResponseText;
  UPDATE #NumberOfFailures SET VAL = VAL + 1;
 END
CreateObjectSuccess:

-- Get number of failures so far
DECLARE @NumberOfFailures AS INT;
SELECT @NumberOfFailures = CONVERT(INT, [VAL]) FROM #NumberOfFailures;

-- Exit the script if no more data is available in #MetricsData
(SELECT 'MessageText', 'Tried to create ' + CONVERT(VARCHAR, (@CurrentDataRow))
+ ' elements created to Metrics model "'
+ (SELECT VAL FROM #MetricsModelName) + '". Number of failures: ' + CONVERT(VARCHAR(10), (@NumberOfFailures))) UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,COUNT(*)) > 0,'0','1') FROM #MetricsData )
--#Exit 

-- Recursively call this script again.
DECLARE @CurrentDataRow AS INT;
IF OBJECT_ID('tempdb..#_Arg1') IS NOT NULL
 SELECT @CurrentDataRow = CONVERT(INT, [CurrentDataRow]) FROM #_Arg1;
ELSE
 SET @CurrentDataRow = 1;

print 'CreateMetricsElements: Current data row: ' + CONVERT(NVARCHAR, @CurrentDataRow)

SET @CurrentDataRow = @CurrentDataRow + 1;

(SELECT 'ScriptId', (SELECT VAL FROM #SubScriptId))
(SELECT @CurrentDataRow AS CurrentDataRow)
--#Run
